@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 1999-2014 Vaclav Slavik
+ *  Copyright (C) 1999-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -35,10 +35,6 @@
 #include <wx/msgdlg.h>
 #include <wx/filename.h>
 
-#ifdef __WXOSX__
-#include <wx/cocoa/string.h>
-#endif
-
 #include <set>
 #include <algorithm>
 
@@ -51,35 +47,24 @@
 #include "version.h"
 #include "language.h"
 
+#ifdef __WXOSX__
+#import <Foundation/Foundation.h>
+#include "osx_helpers.h"
+#endif
 
 // ----------------------------------------------------------------------
 // Textfile processing utilities:
 // ----------------------------------------------------------------------
 
-
-// Read one line from file, remove all \r and \n characters, ignore empty lines
-static wxString ReadTextLine(wxTextFile* f)
+namespace
 {
-    wxString s;
-
-    while (s.empty())
-    {
-        if (f->Eof()) return wxEmptyString;
-
-        // read next line and strip insignificant whitespace from it:
-        s = f->GetNextLine().Strip(wxString::both);
-    }
-
-    return s;
-}
-
 
 // If input begins with pattern, fill output with end of input (without
 // pattern; strips trailing spaces) and return true.  Return false otherwise
 // and don't touch output. Is permissive about whitespace in the input:
 // a space (' ') in pattern will match any number of any whitespace characters
 // on that position in input.
-static bool ReadParam(const wxString& input, const wxString& pattern, wxString& output)
+bool ReadParam(const wxString& input, const wxString& pattern, wxString& output)
 {
     if (input.size() < pattern.size())
         return false;
@@ -132,8 +117,11 @@ bool VerifyFileCharset(const wxTextFile& f, const wxString& filename,
 
     if (f.GetLineCount() != f2.GetLineCount())
     {
-        wxLogError(_("%i lines of file '%s' were not loaded correctly."),
-                   (int)f2.GetLineCount() - (int)f.GetLineCount(),
+        int linesCount = (int)f2.GetLineCount() - (int)f.GetLineCount();
+        wxLogError(wxPLURAL("%i line of file '%s' was not loaded correctly.",
+                            "%i lines of file '%s' were not loaded correctly.",
+                            linesCount),
+                   linesCount,
                    filename.c_str());
         return false;
     }
@@ -156,7 +144,7 @@ bool VerifyFileCharset(const wxTextFile& f, const wxString& filename,
 
 
 // converts \n into newline character and \\ into \:
-static wxString UnescapeCEscapes(const wxString& str)
+wxString UnescapeCEscapes(const wxString& str)
 {
     wxString out;
     size_t len = str.size();
@@ -201,6 +189,42 @@ static wxString UnescapeCEscapes(const wxString& str)
 }
 
 
+wxTextFileType GetFileCRLFFormat(wxTextFile& po_file)
+{
+    wxLogNull null;
+    auto crlf = po_file.GuessType();
+
+    // Discard any unsupported setting. In particular, we ignore "Mac"
+    // line endings, because the ancient OS 9 systems aren't used anymore,
+    // OSX uses Unix ending *and* "Mac" endings break gettext tools. So if
+    // we encounter a catalog with "Mac" line endings, we silently convert
+    // it into Unix endings (i.e. the modern Mac).
+    if (crlf == wxTextFileType_Mac)
+        crlf = wxTextFileType_Unix;
+    if (crlf != wxTextFileType_Dos && crlf != wxTextFileType_Unix)
+        crlf = wxTextFileType_None;
+    return crlf;
+}
+
+wxTextFileType GetDesiredCRLFFormat(wxTextFileType existingCRLF)
+{
+    if (existingCRLF != wxTextFileType_None && wxConfigBase::Get()->ReadBool("keep_crlf", true))
+    {
+        return existingCRLF;
+    }
+    else
+    {
+        wxString format = wxConfigBase::Get()->Read("crlf_format", "unix");
+        if (format == "win")
+            return wxTextFileType_Dos;
+        else /* "unix" or obsolete settings */
+            return wxTextFileType_Unix;
+    }
+}
+
+} // anonymous namespace
+
+
 // ----------------------------------------------------------------------
 // Catalog::HeaderData
 // ----------------------------------------------------------------------
@@ -260,10 +284,18 @@ void Catalog::HeaderData::UpdateDict()
     SetHeader("PO-Revision-Date", RevisionDate);
 
     if (TranslatorEmail.empty())
-        SetHeader("Last-Translator", Translator);
+    {
+        if (!Translator.empty() || !HasHeader("Last-Translator"))
+            SetHeader("Last-Translator", Translator);
+        // else: don't modify the header, leave as-is
+    }
     else
-        SetHeader("Last-Translator",
-                  Translator + " <" + TranslatorEmail + ">");
+    {
+        if (Translator.empty())
+            SetHeader("Last-Translator", TranslatorEmail);
+        else
+            SetHeader("Last-Translator", Translator + " <" + TranslatorEmail + ">");
+    }
 
     if (TeamEmail.empty())
         SetHeader("Language-Team", Team);
@@ -570,7 +602,7 @@ bool CatalogParser::Parse()
 
     wxString line, dummy;
     wxString mflags, mstr, msgid_plural, mcomment;
-    wxArrayString mrefs, mautocomments, mtranslations;
+    wxArrayString mrefs, mextractedcomments, mtranslations;
     wxArrayString msgid_old;
     bool has_plural = false;
     bool has_context = false;
@@ -578,28 +610,28 @@ bool CatalogParser::Parse()
     unsigned mlinenum = 0;
 
     line = m_textFile->GetFirstLine();
-    if (line.empty()) line = ReadTextLine(m_textFile);
+    if (line.empty()) line = ReadTextLine();
 
     while (!line.empty())
     {
-        // ignore empty special tags (except for automatic comments which we
+        // ignore empty special tags (except for extracted comments which we
         // DO want to preserve):
         while (line == "#," || line == "#:" || line == "#|")
-            line = ReadTextLine(m_textFile);
+            line = ReadTextLine();
 
         // flags:
         // Can't we have more than one flag, now only the last is kept ...
         if (ReadParam(line, "#, ", dummy))
         {
             mflags = "#, " + dummy;
-            line = ReadTextLine(m_textFile);
+            line = ReadTextLine();
         }
 
         // auto comments:
         if (ReadParam(line, "#. ", dummy) || ReadParam(line, "#.", dummy)) // second one to account for empty auto comments
         {
-            mautocomments.Add(dummy);
-            line = ReadTextLine(m_textFile);
+            mextractedcomments.Add(dummy);
+            line = ReadTextLine();
         }
 
         // references:
@@ -608,14 +640,14 @@ bool CatalogParser::Parse()
             // Just store the references unmodified, we don't modify this
             // data anywhere.
             mrefs.push_back(dummy);
-            line = ReadTextLine(m_textFile);
+            line = ReadTextLine();
         }
 
         // previous msgid value:
         else if (ReadParam(line, "#| ", dummy))
         {
             msgid_old.Add(dummy);
-            line = ReadTextLine(m_textFile);
+            line = ReadTextLine();
         }
 
         // msgctxt:
@@ -623,12 +655,15 @@ bool CatalogParser::Parse()
         {
             has_context = true;
             msgctxt = dummy.RemoveLast();
-            while (!(line = ReadTextLine(m_textFile)).empty())
+            while (!(line = ReadTextLine()).empty())
             {
                 if (line[0u] == _T('\t'))
                     line.Remove(0, 1);
                 if (line[0u] == _T('"') && line.Last() == _T('"'))
+                {
                     msgctxt += line.Mid(1, line.Length() - 2);
+                    PossibleWrappedLine();
+                }
                 else
                     break;
             }
@@ -639,12 +674,15 @@ bool CatalogParser::Parse()
         {
             mstr = dummy.RemoveLast();
             mlinenum = unsigned(m_textFile->GetCurrentLine() + 1);
-            while (!(line = ReadTextLine(m_textFile)).empty())
+            while (!(line = ReadTextLine()).empty())
             {
                 if (line[0u] == _T('\t'))
                     line.Remove(0, 1);
                 if (line[0u] == _T('"') && line.Last() == _T('"'))
+                {
                     mstr += line.Mid(1, line.Length() - 2);
+                    PossibleWrappedLine();
+                }
                 else
                     break;
             }
@@ -656,12 +694,15 @@ bool CatalogParser::Parse()
             msgid_plural = dummy.RemoveLast();
             has_plural = true;
             mlinenum = unsigned(m_textFile->GetCurrentLine() + 1);
-            while (!(line = ReadTextLine(m_textFile)).empty())
+            while (!(line = ReadTextLine()).empty())
             {
                 if (line[0u] == _T('\t'))
                     line.Remove(0, 1);
                 if (line[0u] == _T('"') && line.Last() == _T('"'))
+                {
                     msgid_plural += line.Mid(1, line.Length() - 2);
+                    PossibleWrappedLine();
+                }
                 else
                     break;
             }
@@ -677,12 +718,15 @@ bool CatalogParser::Parse()
             }
 
             wxString str = dummy.RemoveLast();
-            while (!(line = ReadTextLine(m_textFile)).empty())
+            while (!(line = ReadTextLine()).empty())
             {
                 if (line[0u] == _T('\t'))
                     line.Remove(0, 1);
                 if (line[0u] == _T('"') && line.Last() == _T('"'))
+                {
                     str += line.Mid(1, line.Length() - 2);
+                    PossibleWrappedLine();
+                }
                 else
                     break;
             }
@@ -701,7 +745,7 @@ bool CatalogParser::Parse()
                 if (!OnEntry(mstr, wxEmptyString, false,
                              has_context, msgctxt,
                              mtranslations,
-                             mflags, mrefs, mcomment, mautocomments, msgid_old,
+                             mflags, mrefs, mcomment, mextractedcomments, msgid_old,
                              mlinenum))
                 {
                     return false;
@@ -711,7 +755,7 @@ bool CatalogParser::Parse()
             mcomment = mstr = msgid_plural = msgctxt = mflags = wxEmptyString;
             has_plural = has_context = false;
             mrefs.Clear();
-            mautocomments.Clear();
+            mextractedcomments.Clear();
             mtranslations.Clear();
             msgid_old.Clear();
         }
@@ -732,11 +776,14 @@ bool CatalogParser::Parse()
             {
                 wxString str = dummy.RemoveLast();
 
-                while (!(line=ReadTextLine(m_textFile)).empty())
+                while (!(line=ReadTextLine()).empty())
                 {
                     line.Trim(/*fromRight=*/false);
                     if (line[0u] == _T('"') && line.Last() == _T('"'))
+                    {
                         str += line.Mid(1, line.Length() - 2);
+                        PossibleWrappedLine();
+                    }
                     else
                     {
                         if (ReadParam(line, "msgstr[", dummy))
@@ -753,7 +800,7 @@ bool CatalogParser::Parse()
             if (!OnEntry(mstr, msgid_plural, true,
                          has_context, msgctxt,
                          mtranslations,
-                         mflags, mrefs, mcomment, mautocomments, msgid_old,
+                         mflags, mrefs, mcomment, mextractedcomments, msgid_old,
                          mlinenum))
             {
                 return false;
@@ -762,7 +809,7 @@ bool CatalogParser::Parse()
             mcomment = mstr = msgid_plural = msgctxt = mflags = wxEmptyString;
             has_plural = has_context = false;
             mrefs.Clear();
-            mautocomments.Clear();
+            mextractedcomments.Clear();
             mtranslations.Clear();
             msgid_old.Clear();
         }
@@ -773,7 +820,7 @@ bool CatalogParser::Parse()
             wxArrayString deletedLines;
             deletedLines.Add(line);
             mlinenum = unsigned(m_textFile->GetCurrentLine() + 1);
-            while (!(line = ReadTextLine(m_textFile)).empty())
+            while (!(line = ReadTextLine()).empty())
             {
                 // if line does not start with "#~" anymore, stop reading
                 if (!ReadParam(line, "#~", dummy))
@@ -786,7 +833,7 @@ bool CatalogParser::Parse()
                 deletedLines.Add(line);
             }
             if (!OnDeletedEntry(deletedLines,
-                                mflags, mrefs, mcomment, mautocomments, mlinenum))
+                                mflags, mrefs, mcomment, mextractedcomments, mlinenum))
             {
                 return false;
             }
@@ -794,7 +841,7 @@ bool CatalogParser::Parse()
             mcomment = mstr = msgid_plural = mflags = wxEmptyString;
             has_plural = false;
             mrefs.Clear();
-            mautocomments.Clear();
+            mextractedcomments.Clear();
             mtranslations.Clear();
             msgid_old.Clear();
         }
@@ -810,16 +857,16 @@ bool CatalogParser::Parse()
             {
                 mcomment << line << _T('\n');
                 readNewLine = true;
-                line = ReadTextLine(m_textFile);
+                line = ReadTextLine();
             }
 
             if (!readNewLine)
-                line = ReadTextLine(m_textFile);
+                line = ReadTextLine();
         }
 
         else
         {
-            line = ReadTextLine(m_textFile);
+            line = ReadTextLine();
         }
     }
 
@@ -827,6 +874,63 @@ bool CatalogParser::Parse()
 }
 
 
+wxString CatalogParser::ReadTextLine()
+{
+    m_previousLineHardWrapped = m_lastLineHardWrapped;
+    m_lastLineHardWrapped = false;
+
+    wxString s;
+
+    while (s.empty())
+    {
+        if (m_textFile->Eof())
+            return wxEmptyString;
+
+        // read next line and strip insignificant whitespace from it:
+        auto ln = m_textFile->GetNextLine();
+
+        // gettext tools don't include extracted comments in wrapping, so they can't
+        // be reliably used to detect file's wrapping either; just skip them.
+        if (!ln.StartsWith(wxS("#. ")))
+        {
+            if (ln.EndsWith(wxS("\\n\"")))
+            {
+                // Similarly, lines ending with \n are always wrapped, so skip that too.
+                m_lastLineHardWrapped = true;
+            }
+            else if (ln == "msgid \"\"" || ln == "msgstr \"\"")
+            {
+                // The header is always indented like this
+                m_lastLineHardWrapped = true;
+            }
+            else
+            {
+                // Watch out for lines with too long words that couldn't be wrapped.
+                // That "2" is to account for unwrappable comment lines: "#: somethinglong"
+                // See https://github.com/vslavik/poedit/issues/135
+                auto space = ln.find_last_of(' ');
+                if (space != wxString::npos && space > 2)
+                {
+                    m_detectedLineWidth = std::max(m_detectedLineWidth, (int)ln.size());
+                }
+            }
+        }
+
+        s = ln.Strip(wxString::both);
+    }
+
+    return s;
+}
+
+int CatalogParser::GetWrappingWidth() const
+{
+    if (!m_detectedWrappedLines)
+        return Catalog::NO_WRAPPING;
+
+    return m_detectedLineWidth;
+}
+
+
 
 class CharsetInfoFinder : public CatalogParser
 {
@@ -847,7 +951,7 @@ class CharsetInfoFinder : public CatalogParser
                              const wxString& /*flags*/,
                              const wxArrayString& /*references*/,
                              const wxString& /*comment*/,
-                             const wxArrayString& /*autocomments*/,
+                             const wxArrayString& /*extractedComments*/,
                              const wxArrayString& /*msgid_old*/,
                              unsigned /*lineNumber*/)
         {
@@ -890,7 +994,7 @@ class LoadParser : public CatalogParser
                              const wxString& flags,
                              const wxArrayString& references,
                              const wxString& comment,
-                             const wxArrayString& autocomments,
+                             const wxArrayString& extractedComments,
                              const wxArrayString& msgid_old,
                              unsigned lineNumber);
 
@@ -898,7 +1002,7 @@ class LoadParser : public CatalogParser
                                     const wxString& flags,
                                     const wxArrayString& references,
                                     const wxString& comment,
-                                    const wxArrayString& autocomments,
+                                    const wxArrayString& extractedComments,
                                     unsigned lineNumber);
 
         virtual void OnIgnoredEntry() { FileIsValid = true; }
@@ -918,7 +1022,7 @@ bool LoadParser::OnEntry(const wxString& msgid,
                          const wxString& flags,
                          const wxArrayString& references,
                          const wxString& comment,
-                         const wxArrayString& autocomments,
+                         const wxArrayString& extractedComments,
                          const wxArrayString& msgid_old,
                          unsigned lineNumber)
 {
@@ -954,15 +1058,15 @@ bool LoadParser::OnEntry(const wxString& msgid,
         for (size_t i = 0; i < references.GetCount(); i++)
             d.AddReference(references[i]);
 
-        for (auto i: autocomments)
+        for (auto i: extractedComments)
         {
-            // Sometimes, msgcat produces conflicts in automatic comments; see the gory details:
+            // Sometimes, msgcat produces conflicts in extracted comments; see the gory details:
             // https://groups.google.com/d/topic/poedit/j41KuvXtVUU/discussion
             // As a workaround, just filter them out.
             // FIXME: Fix this properly... but not using msgcat in the first place
             if (i.StartsWith(MSGCAT_CONFLICT_MARKER) && i.EndsWith(MSGCAT_CONFLICT_MARKER))
                 continue;
-            d.AddAutoComments(i);
+            d.AddExtractedComments(i);
         }
         d.SetOldMsgid(msgid_old);
         m_catalog->AddItem(d);
@@ -974,7 +1078,7 @@ bool LoadParser::OnDeletedEntry(const wxArrayString& deletedLines,
                                 const wxString& flags,
                                 const wxArrayString& /*references*/,
                                 const wxString& comment,
-                                const wxArrayString& autocomments,
+                                const wxArrayString& extractedComments,
                                 unsigned lineNumber)
 {
     FileIsValid = true;
@@ -984,8 +1088,8 @@ bool LoadParser::OnDeletedEntry(const wxArrayString& deletedLines,
     d.SetDeletedLines(deletedLines);
     d.SetComment(comment);
     d.SetLineNumber(lineNumber);
-    for (size_t i = 0; i < autocomments.GetCount(); i++)
-      d.AddAutoComments(autocomments[i]);
+    for (size_t i = 0; i < extractedComments.GetCount(); i++)
+      d.AddExtractedComments(extractedComments[i]);
     m_catalog->AddDeletedItem(d);
 
     return true;
@@ -999,6 +1103,9 @@ bool LoadParser::OnDeletedEntry(const wxArrayString& deletedLines,
 
 Catalog::Catalog()
 {
+    m_fileCRLF = wxTextFileType_None;
+    m_fileWrappingWidth = DEFAULT_WRAPPING;
+
     m_isOk = true;
     m_header.BasePath = wxEmptyString;
     for(int i = BOOKMARK_0; i < BOOKMARK_LAST; i++)
@@ -1016,6 +1123,9 @@ Catalog::~Catalog()
 
 Catalog::Catalog(const wxString& po_file, int flags)
 {
+    m_fileCRLF = wxTextFileType_None;
+    m_fileWrappingWidth = DEFAULT_WRAPPING;
+
     m_isOk = Load(po_file, flags);
 }
 
@@ -1130,6 +1240,10 @@ bool Catalog::Load(const wxString& po_file, int flags)
         }
     }
 
+    m_fileCRLF = GetFileCRLFFormat(f);
+    m_fileWrappingWidth = parser.GetWrappingWidth();
+    wxLogTrace("poedit", "detect line wrapping: %d", m_fileWrappingWidth);
+
     // If we didn't find any entries, the file must be invalid:
     if (!parser.FileIsValid)
         return false;
@@ -1161,7 +1275,21 @@ void Catalog::FixupCommonIssues()
     wxLogTrace("poedit", "catalog lang is '%s'", GetLanguage().Code());
 
     if (m_header.Project == "PACKAGE VERSION")
-        m_header.Project = "";
+        m_header.Project.clear();
+
+    if (m_header.GetHeader("Language-Team") == "LANGUAGE <LL@li.org>")
+    {
+        m_header.DeleteHeader("Language-Team");
+        m_header.Team.clear();
+        m_header.TeamEmail.clear();
+    }
+
+    if (m_header.GetHeader("Last-Translator") == "FULL NAME <EMAIL@ADDRESS>")
+    {
+        m_header.DeleteHeader("Last-Translator");
+        m_header.Translator.clear();
+        m_header.TranslatorEmail.clear();
+    }
 
     wxString pluralForms = m_header.GetHeader("Plural-Forms");
 
@@ -1289,51 +1417,6 @@ bool CanEncodeToCharset(const wxTextFile& f, const wxString& charset)
 }
 
 
-void GetCRLFBehaviour(wxTextFileType& type, bool& preserve)
-{
-    wxString format = wxConfigBase::Get()->Read("crlf_format", "unix");
-
-    if (format == "win") type = wxTextFileType_Dos;
-    else /* "unix" or obsolete settings */ type = wxTextFileType_Unix;
-
-    preserve = wxConfigBase::Get()->ReadBool("keep_crlf", true);
-}
-
-
-wxTextFileType GetDesiredCRLFFormat(const wxString& po_file)
-{
-    wxTextFileType crlfDefault, crlf;
-    bool crlfPreserve;
-    GetCRLFBehaviour(crlfDefault, crlfPreserve);
-
-    wxTextFile f;
-    if ( crlfPreserve && wxFileExists(po_file) &&
-         f.Open(po_file, wxConvISO8859_1) )
-    {
-        wxLogNull null;
-        crlf = f.GuessType();
-
-        // Discard any unsupported setting. In particular, we ignore "Mac"
-        // line endings, because the ancient OS 9 systems aren't used anymore,
-        // OSX uses Unix ending *and* "Mac" endings break gettext tools. So if
-        // we encounter a catalog with "Mac" line endings, we silently convert
-        // it into Unix endings (i.e. the modern Mac).
-        if (crlf == wxTextFileType_Mac)
-            crlf = wxTextFileType_Unix;
-        if (crlf != wxTextFileType_Dos && crlf != wxTextFileType_Unix)
-            crlf = crlfDefault;
-
-        f.Close();
-    }
-    else
-    {
-        crlf = crlfDefault;
-    }
-
-    return crlf;
-}
-
-
 void SaveMultiLines(wxTextFile &f, const wxString& text)
 {
     wxStringTokenizer tkn(text, _T('\n'));
@@ -1384,8 +1467,8 @@ wxString FormatStringForFile(const wxString& text)
 #ifdef __WXOSX__
 
 @interface CompiledMOFilePresenter : NSObject<NSFilePresenter>
-@property (atomic, strong) NSURL *presentedItemURL;
-@property (atomic, strong) NSURL *primaryPresentedItemURL;
+@property (atomic, copy) NSURL *presentedItemURL;
+@property (atomic, copy) NSURL *primaryPresentedItemURL;
 @end
 
 @implementation CompiledMOFilePresenter
@@ -1409,17 +1492,15 @@ bool Catalog::Save(const wxString& po_file, bool save_mo,
         return false;
     }
 
-    // Update information about last modification time. But if the header
-    // was empty previously, the author apparently doesn't want this header
-    // set, so don't mess with it. See https://sourceforge.net/tracker/?func=detail&atid=389156&aid=1900298&group_id=27043
-    // for motivation:
-    if ( !m_header.RevisionDate.empty() )
-        m_header.RevisionDate = GetCurrentTimeRFC822();
-
     TempOutputFileFor po_file_temp_obj(po_file);
     const wxString po_file_temp = po_file_temp_obj.FileName();
 
-    if ( !DoSaveOnly(po_file_temp) )
+    wxTextFileType outputCrlf = GetDesiredCRLFFormat(m_fileCRLF);
+    // Save into Unix line endings first and only if Windows is required,
+    // reformat the file later. This is because msgcat cannot handle DOS
+    // input particularly well.
+
+    if ( !DoSaveOnly(po_file_temp, wxTextFileType_Unix) )
     {
         wxLogError(_("Couldn't save file %s."), po_file.c_str());
         return false;
@@ -1433,20 +1514,41 @@ bool Catalog::Save(const wxString& po_file, bool save_mo,
 
     int msgcat_ok = false;
     {
+        int wrapping = DEFAULT_WRAPPING;
+        if (wxConfig::Get()->ReadBool("keep_crlf", true))
+            wrapping = m_fileWrappingWidth;
+
+        wxString wrappingFlag;
+        if (wrapping == DEFAULT_WRAPPING)
+        {
+            if (wxConfig::Get()->ReadBool("wrap_po_files", true))
+            {
+                wrapping = (int)wxConfig::Get()->ReadLong("wrap_po_files_width", 79);
+            }
+            else
+            {
+                wrapping = NO_WRAPPING;
+            }
+        }
+
+        if (wrapping == NO_WRAPPING)
+            wrappingFlag = " --no-wrap";
+        else if (wrapping != DEFAULT_WRAPPING)
+            wrappingFlag.Printf(" --width=%d", wrapping);
+
+        auto msgcatCmd = wxString::Format("msgcat --force-po%s -o %s %s",
+                                          wrappingFlag,
+                                          QuoteCmdlineArg(po_file),
+                                          QuoteCmdlineArg(po_file_temp));
+        wxLogTrace("poedit", "formatting file with %s", msgcatCmd);
+
         // Ignore msgcat errors output (but not exit code), because it
         //   a) complains about things DoValidate() already complained above
         //   b) issues warnings about source-extraction things (e.g. using non-ASCII
         //      msgids) that, while correct, are not something a *translator* can
         //      do anything about.
         wxLogNull null;
-        msgcat_ok =
-              ExecuteGettext
-              (
-                  wxString::Format("msgcat --force-po -o %s %s",
-                                   QuoteCmdlineArg(po_file),
-                                   QuoteCmdlineArg(po_file_temp))
-              )
-              && wxFileExists(po_file);
+        msgcat_ok = ExecuteGettext(msgcatCmd) && wxFileExists(po_file);
     }
 
     if ( msgcat_ok )
@@ -1471,11 +1573,21 @@ bool Catalog::Save(const wxString& po_file, bool save_mo,
         }
     }
 
+    
+    // msgcat always outputs Unix line endings, so we need to reformat the file
+    if (outputCrlf == wxTextFileType_Dos)
+    {
+        wxTextFile finalFile(po_file);
+        if (finalFile.Open())
+            finalFile.Write(outputCrlf);
+    }
+
+
     /* If the user wants it, compile .mo file right now: */
 
     if (save_mo && wxConfig::Get()->Read("compile_mo", (long)true))
     {
-        const wxString mo_file = po_file.BeforeLast('.') + ".mo";
+        const wxString mo_file = wxFileName::StripExtension(po_file) + ".mo";
         TempOutputFileFor mo_file_temp_obj(mo_file);
         const wxString mo_file_temp = mo_file_temp_obj.FileName();
 
@@ -1514,13 +1626,18 @@ bool Catalog::Save(const wxString& po_file, bool save_mo,
         if (mo_compilation_status == CompilationStatus::Success)
         {
 #ifdef __WXOSX__
-            CompiledMOFilePresenter *presenter = [CompiledMOFilePresenter new];
-            NSURL *mofileUrl = [NSURL fileURLWithPath:[NSString stringWithUTF8String: mo_file.utf8_str()]];
-            NSURL *mofiletempUrl = [NSURL fileURLWithPath:[NSString stringWithUTF8String: mo_file_temp.utf8_str()]];
-            presenter.presentedItemURL = mofileUrl;
-            presenter.primaryPresentedItemURL = [NSURL fileURLWithPath:[NSString stringWithUTF8String: po_file.utf8_str()]];
-            [NSFileCoordinator addFilePresenter:presenter];
-            [NSFileCoordinator filePresenters];
+            NSURL *mofileUrl = [NSURL fileURLWithPath:wxStringToNS(mo_file)];
+            NSURL *mofiletempUrl = [NSURL fileURLWithPath:wxStringToNS(mo_file_temp)];
+            bool sandboxed = (getenv("APP_SANDBOX_CONTAINER_ID") != NULL);
+            CompiledMOFilePresenter *presenter = nil;
+            if (sandboxed)
+            {
+                presenter = [CompiledMOFilePresenter new];
+                presenter.presentedItemURL = mofileUrl;
+                presenter.primaryPresentedItemURL = [NSURL fileURLWithPath:wxStringToNS(po_file)];
+                [NSFileCoordinator addFilePresenter:presenter];
+                [NSFileCoordinator filePresenters];
+            }
             NSFileCoordinator *coo = [[NSFileCoordinator alloc] initWithFilePresenter:presenter];
             [coo coordinateWritingItemAtURL:mofileUrl options:NSFileCoordinatorWritingForReplacing error:nil byAccessor:^(NSURL *newURL) {
                 NSURL *resultingUrl;
@@ -1536,7 +1653,10 @@ bool Catalog::Save(const wxString& po_file, bool save_mo,
                     mo_compilation_status = CompilationStatus::Error;
                 }
             }];
-            [NSFileCoordinator removeFilePresenter:presenter];
+            if (sandboxed)
+            {
+                [NSFileCoordinator removeFilePresenter:presenter];
+            }
 #else // !__WXOSX__
             if ( !wxRenameFile(mo_file_temp, mo_file, /*overwrite=*/true) )
             {
@@ -1552,11 +1672,65 @@ bool Catalog::Save(const wxString& po_file, bool save_mo,
     return true;
 }
 
-
-bool Catalog::DoSaveOnly(const wxString& po_file)
+bool Catalog::CompileToMO(const wxString& mo_file,
+                          int& validation_errors,
+                          CompilationStatus& mo_compilation_status)
 {
-    wxTextFileType crlf = GetDesiredCRLFFormat(po_file);
+    mo_compilation_status = CompilationStatus::NotDone;
 
+    TempDirectory tmpdir;
+    if ( !tmpdir.IsOk() )
+        return false;
+    wxString po_file_temp = tmpdir.CreateFileName("output.po");
+
+    if ( !DoSaveOnly(po_file_temp, wxTextFileType_Unix) )
+    {
+        wxLogError(_("Couldn't save file %s."), po_file_temp.c_str());
+        return false;
+    }
+
+    validation_errors = DoValidate(po_file_temp);
+
+    TempOutputFileFor mo_file_temp_obj(mo_file);
+    const wxString mo_file_temp = mo_file_temp_obj.FileName();
+
+    {
+        // Ignore msgfmt errors output (but not exit code), because it
+        // complains about things DoValidate() already complained above.
+        wxLogNull null;
+        ExecuteGettext(wxString::Format("msgfmt -o %s %s",
+                                        QuoteCmdlineArg(mo_file_temp),
+                                        QuoteCmdlineArg(po_file_temp)));
+    }
+
+    // Don't check return code:
+    // msgfmt has the ugly habit of sometimes returning non-zero
+    // exit code, reporting "fatal errors" and *still* producing a usable
+    // .mo file. If this happens, don't pretend the file wasn't created.
+    if (!wxFileName::FileExists(mo_file_temp))
+    {
+        mo_compilation_status = CompilationStatus::Error;
+        return false;
+    }
+    else
+    {
+        mo_compilation_status = CompilationStatus::Success;
+    }
+
+    if ( !wxRenameFile(mo_file_temp, mo_file, /*overwrite=*/true) )
+    {
+        wxLogError(_("Couldn't save file %s."), mo_file.c_str());
+        return false;
+    }
+
+    return true;
+}
+
+
+
+
+bool Catalog::DoSaveOnly(const wxString& po_file, wxTextFileType crlf)
+{
     wxTextFile f;
     if (!f.Create(po_file))
         return false;
@@ -1570,6 +1744,13 @@ bool Catalog::DoSaveOnly(wxTextFile& f, wxTextFileType crlf)
     if (!m_header.Charset || m_header.Charset == "CHARSET")
         m_header.Charset = "UTF-8";
 
+    // Update information about last modification time. But if the header
+    // was empty previously, the author apparently doesn't want this header
+    // set, so don't mess with it. See https://sourceforge.net/tracker/?func=detail&atid=389156&aid=1900298&group_id=27043
+    // for motivation:
+    if ( !m_header.RevisionDate.empty() )
+        m_header.RevisionDate = GetCurrentTimeRFC822();
+
     SaveMultiLines(f, m_header.Comment);
     f.AddLine(_T("msgid \"\""));
     f.AddLine(_T("msgstr \"\""));
@@ -1583,12 +1764,12 @@ bool Catalog::DoSaveOnly(wxTextFile& f, wxTextFileType crlf)
         CatalogItem& data = m_items[itemIdx];
         data.SetLineNumber(int(f.GetLineCount()+1));
         SaveMultiLines(f, data.GetComment());
-        for (unsigned i = 0; i < data.GetAutoComments().GetCount(); i++)
+        for (unsigned i = 0; i < data.GetExtractedComments().GetCount(); i++)
         {
-            if (data.GetAutoComments()[i].empty())
+            if (data.GetExtractedComments()[i].empty())
               f.AddLine("#.");
             else
-              f.AddLine("#. " + data.GetAutoComments()[i]);
+              f.AddLine("#. " + data.GetExtractedComments()[i]);
         }
         for (unsigned i = 0; i < data.GetRawReferences().GetCount(); i++)
             f.AddLine("#: " + data.GetRawReferences()[i]);
@@ -1632,8 +1813,8 @@ bool Catalog::DoSaveOnly(wxTextFile& f, wxTextFileType crlf)
         CatalogDeletedData& deletedItem = m_deletedItems[itemIdx];
         deletedItem.SetLineNumber(int(f.GetLineCount()+1));
         SaveMultiLines(f, deletedItem.GetComment());
-        for (unsigned i = 0; i < deletedItem.GetAutoComments().GetCount(); i++)
-            f.AddLine("#. " + deletedItem.GetAutoComments()[i]);
+        for (unsigned i = 0; i < deletedItem.GetExtractedComments().GetCount(); i++)
+            f.AddLine("#. " + deletedItem.GetExtractedComments()[i]);
         for (unsigned i = 0; i < deletedItem.GetRawReferences().GetCount(); i++)
             f.AddLine("#: " + deletedItem.GetRawReferences()[i]);
         wxString dummy = deletedItem.GetFlags();
@@ -1670,7 +1851,7 @@ int Catalog::Validate()
         return 0;
 
     wxString tmp_po = tmpdir.CreateFileName("validated.po");
-    if ( !DoSaveOnly(tmp_po) )
+    if ( !DoSaveOnly(tmp_po, wxTextFileType_Unix) )
         return 0;
 
     return DoValidate(tmp_po);
@@ -1711,9 +1892,12 @@ int Catalog::DoValidate(const wxString& po_file)
 }
 
 
-bool Catalog::Update(ProgressInfo *progress, bool summary, bool *cancelledByUser)
+bool Catalog::Update(ProgressInfo *progress, bool summary, UpdateResultReason& reason)
 {
-    if (!m_isOk) return false;
+    reason = UpdateResultReason::Unspecified;
+
+    if (!m_isOk)
+        return false;
 
     wxString cwd = wxGetCwd();
     if (m_fileName != wxEmptyString)
@@ -1737,7 +1921,7 @@ bool Catalog::Update(ProgressInfo *progress, bool summary, bool *cancelledByUser
 
         if (!wxFileName::DirExists(path))
         {
-            wxLogError(_("Source code directory '%s' doesn't exist."), path.c_str());
+            reason = UpdateResultReason::NoSourcesFound;
             return false;
         }
 
@@ -1749,7 +1933,11 @@ bool Catalog::Update(ProgressInfo *progress, bool summary, bool *cancelledByUser
     Catalog *newcat = dig.Dig(m_header.SearchPaths,
                               m_header.SearchPathsExcluded,
                               m_header.Keywords,
-                              m_header.SourceCodeCharset);
+                              m_header.SourceCodeCharset,
+                              reason);
+
+    if (progress->Cancelled())
+        reason = UpdateResultReason::CancelledByUser;
 
     if (newcat != NULL)
     {
@@ -1757,13 +1945,16 @@ bool Catalog::Update(ProgressInfo *progress, bool summary, bool *cancelledByUser
         if ( progress )
             progress->UpdateMessage(_("Merging differences..."));
 
-        if (!summary || ShowMergeSummary(newcat, cancelledByUser))
+        bool cancelledByUser = false;
+        if (!summary || ShowMergeSummary(newcat, &cancelledByUser))
             succ = Merge(newcat);
         if (!succ)
         {
             delete newcat;
             newcat = NULL;
         }
+        if (cancelledByUser)
+            reason = UpdateResultReason::CancelledByUser;
     }
 
     wxSetWorkingDirectory(cwd);
@@ -1776,9 +1967,11 @@ bool Catalog::Update(ProgressInfo *progress, bool summary, bool *cancelledByUser
 
 
 bool Catalog::UpdateFromPOT(const wxString& pot_file,
-                            bool summary, bool *cancelledByUser,
+                            bool summary,
+                            UpdateResultReason& reason,
                             bool replace_header)
 {
+    reason = UpdateResultReason::Unspecified;
     if (!m_isOk) return false;
 
     Catalog newcat(pot_file, CreationFlag_IgnoreTranslations);
@@ -1789,7 +1982,8 @@ bool Catalog::UpdateFromPOT(const wxString& pot_file,
         return false;
     }
 
-    if (!summary || ShowMergeSummary(&newcat, cancelledByUser))
+    bool cancelledByUser = false;
+    if (!summary || ShowMergeSummary(&newcat, &cancelledByUser))
     {
         if ( !Merge(&newcat) )
             return false;
@@ -1799,6 +1993,8 @@ bool Catalog::UpdateFromPOT(const wxString& pot_file,
     }
     else
     {
+        if (cancelledByUser)
+            reason = UpdateResultReason::CancelledByUser;
         return false;
     }
 }
@@ -1816,14 +2012,21 @@ bool Catalog::Merge(Catalog *refcat)
     wxString tmp2 = tmpdir.CreateFileName("input.po");
     wxString tmp3 = tmpdir.CreateFileName("output.po");
 
-    refcat->DoSaveOnly(tmp1);
-    DoSaveOnly(tmp2);
+    refcat->DoSaveOnly(tmp1, wxTextFileType_Unix);
+    DoSaveOnly(tmp2, wxTextFileType_Unix);
+
+    wxString flags("-q --force-po");
+    if (wxConfig::Get()->ReadBool("use_tm_when_updating", false) == false)
+    {
+        flags += " --no-fuzzy-matching";
+    }
 
     bool succ = ExecuteGettext
                 (
                     wxString::Format
                     (
-                        "msgmerge -q --force-po -o %s %s %s",
+                        "msgmerge %s -o %s %s %s",
+                        flags,
                         QuoteCmdlineArg(tmp3),
                         QuoteCmdlineArg(tmp2),
                         QuoteCmdlineArg(tmp1)
@@ -2105,6 +2308,47 @@ void CatalogItem::SetTranslations(const wxArrayString &t)
     }
 }
 
+void CatalogItem::SetTranslationFromSource()
+{
+    m_validity = Val_Unknown;
+    m_isFuzzy = false;
+    m_isAutomatic = false;
+    m_isTranslated = true;
+
+    auto iter = m_translations.begin();
+    if (*iter != m_string)
+    {
+        *iter = m_string;
+        m_isModified = true;
+    }
+
+    if (m_hasPlural)
+    {
+        ++iter;
+        for ( ; iter != m_translations.end(); ++iter )
+        {
+            if (*iter != m_plural)
+            {
+                *iter = m_plural;
+                m_isModified = true;
+            }
+        }
+    }
+}
+
+void CatalogItem::ClearTranslation()
+{
+    m_isFuzzy = false;
+    m_isAutomatic = false;
+    m_isTranslated = false;
+    for (auto& t: m_translations)
+    {
+        if (!t.empty())
+            m_isModified = true;
+        t.clear();
+    }
+}
+
 unsigned CatalogItem::GetPluralFormsCount() const
 {
     unsigned trans = GetNumberOfTranslations();
