@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2000-2014 Vaclav Slavik
+ *  Copyright (C) 2000-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -29,16 +29,17 @@
 #include <wx/hash.h>
 #include <wx/dynarray.h>
 
+#include "catalog.h"
 
 class Catalog;
 class wxArrayString;
-class ParsersDB;
-class Parser;
+class ExtractorsDB;
+class Extractor;
 class ProgressInfo;
 class TempDirectory;
 
 /** This class extracts translatable strings from sources.
-    It uses ParsersDB to get information about external programs to
+    It uses ExtractorsDB to get information about external programs to
     call in order to dig information from single file.
  */
 class SourceDigger
@@ -49,9 +50,9 @@ class SourceDigger
 
         /** Scans files for translatable strings and returns Catalog
             instance containing them. All files in input \a paths that 
-            match file extensions in a definition of parser in ParsersDB
-            instance passed to the ctor are proceed by external parser 
-            program (typically, gettext) according to parser definition.
+            match file extensions in a definition of extractor in ExtractorsDB
+            instance passed to the ctor are proceed by external extractor
+            program (typically, gettext) according to extractor definition.
 
             \param paths    list of directories to look in
             \param excludePaths directories to skip
@@ -62,37 +63,78 @@ class SourceDigger
         Catalog *Dig(const wxArrayString& paths,
                      const wxArrayString& excludePaths,
                      const wxArrayString& keywords,
-                     const wxString& charset);
+                     const wxString& charset,
+                     UpdateResultReason& reason);
 
     private:
+        // Path matching with support for wildcards
+        struct PathToMatch
+        {
+            wxString path;
+            bool isWildcard;
+
+            bool MatchesFile(const wxString& fn) const
+            {
+                if (isWildcard)
+                    return wxMatchWild(path, fn);
+                else
+                    return fn == path || fn.StartsWith(path + "/");
+            }
+        };
+
+        class PathsToMatch
+        {
+        public:
+            PathsToMatch() {}
+            explicit PathsToMatch(const wxArrayString& a)
+            {
+                paths.reserve(a.size());
+                for (auto& p: a)
+                    paths.push_back({p, wxIsWild(p)});
+            }
+
+            bool MatchesFile(const wxString& fn) const
+            {
+                for (auto& p: paths)
+                {
+                    if (p.MatchesFile(fn))
+                        return true;
+                }
+                return false;
+            }
+
+        private:
+            std::vector<PathToMatch> paths;
+        };
+
         /** Finds all parsable files. Returned value is a new[]-allocated
             array of wxArrayString objects. n-th string array in returned
-            array holds list of files that can be parsed by n-th parser
+            array holds list of files that can be parsed by n-th extractor
             in \a pdb database.
          */
         wxArrayString *FindFiles(const wxArrayString& paths,
-                                 const wxArrayString& excludePaths,
-                                 ParsersDB& pdb);
+                                 const PathsToMatch& excludePaths,
+                                 ExtractorsDB& pdb);
 
         /** Finds all files in given directory.
             \return false if an error occured.
          */
         int FindInDir(const wxString& dirname,
-                      const wxArrayString& excludePaths,
+                      const PathsToMatch& excludePaths,
                       wxArrayString& files);
 
         /** Digs translatable strings from given files.
             \param outfiles list to which (temporary) file name of extracted
                             catalog will be appended
             \param files    list of files to parse
-            \param parser   parser definition
+            \param extractor   extractor definition
             \param keywords list of keywords that mark translatable strings
             \param charset  source code charset (may be empty)
          */
         bool DigFiles(TempDirectory& tmpdir,
                       wxArrayString& outFiles,
                       const wxArrayString& files,
-                      Parser &parser, const wxArrayString& keywords,
+                      Extractor &extract, const wxArrayString& keywords,
                       const wxString& charset);
 
         ProgressInfo *m_progressInfo;
