@@ -1,7 +1,7 @@
 ﻿/*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 1999-2014 Vaclav Slavik
+ *  Copyright (C) 1999-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -44,7 +44,6 @@
 
 #ifdef __WXOSX__
 #include "osx_helpers.h"
-#include <wx/cocoa/string.h>
 #endif
 
 #ifdef __WXMSW__
@@ -65,8 +64,10 @@
 #include "edframe.h"
 #include "manager.h"
 #include "prefsdlg.h"
-#include "parser.h"
+#include "extractor.h"
 #include "chooselang.h"
+#include "customcontrols.h"
+#include "hidpi.h"
 #include "icons.h"
 #include "version.h"
 #include "tm/transmem.h"
@@ -126,6 +127,8 @@ bool PoeditApp::OnInit()
     if (!wxApp::OnInit())
         return false;
 
+    InitHiDPIHandling();
+
 #ifdef __WXOSX__
     MoveToApplicationsFolderIfNecessary();
 
@@ -137,7 +140,7 @@ bool PoeditApp::OnInit()
     // timestamps on the logs are of not use for Poedit:
     wxLog::DisableTimestamp();
 
-#if defined(__UNIX__) && !defined(__WXMAC__)
+#if defined(__UNIX__) && !defined(__WXOSX__)
     wxString installPrefix = POEDIT_PREFIX;
 #ifdef __linux__
     wxFileName::SplitPath(wxStandardPaths::Get().GetExecutablePath(), &installPrefix, nullptr, nullptr);
@@ -179,7 +182,7 @@ bool PoeditApp::OnInit()
     SetVendorName("Vaclav Slavik");
     SetAppName("Poedit");
 
-#if defined(__WXMAC__)
+#if defined(__WXOSX__)
     #define CFG_FILE (wxStandardPaths::Get().GetUserConfigDir() + "/net.poedit.Poedit.cfg")
 #elif defined(__UNIX__)
     #define CFG_FILE configFile
@@ -187,7 +190,7 @@ bool PoeditApp::OnInit()
     #define CFG_FILE wxEmptyString
 #endif
 
-#ifdef __WXMAC__
+#ifdef __WXOSX__
     // upgrade from the old location of config file:
     wxString oldcfgfile = wxStandardPaths::Get().GetUserConfigDir() + "/poedit.cfg";
     if (wxFileExists(oldcfgfile) && !wxFileExists(CFG_FILE))
@@ -206,8 +209,9 @@ bool PoeditApp::OnInit()
     wxImage::AddHandler(new wxICOHandler);
 #endif
     wxXmlResource::Get()->InitAllHandlers();
+    wxXmlResource::Get()->AddHandler(new LearnMoreLinkXmlHandler);
 
-#if defined(__WXMAC__)
+#if defined(__WXOSX__)
     wxXmlResource::Get()->LoadAllFiles(wxStandardPaths::Get().GetResourcesDir());
 #elif defined(__WXMSW__)
 	wxStandardPaths::Get().DontIgnoreAppSubDir();
@@ -218,17 +222,17 @@ bool PoeditApp::OnInit()
 
     SetDefaultCfg(wxConfig::Get());
 
-#if defined(__WXMAC__) || defined(__WXMSW__)
+#if defined(__WXOSX__) || defined(__WXMSW__)
     u_setDataDirectory(wxStandardPaths::Get().GetResourcesDir().mb_str());
 #endif
 
-#ifndef __WXMAC__
+#ifndef __WXOSX__
     wxArtProvider::PushBack(new PoeditArtProvider);
 #endif
 
     SetupLanguage();
 
-#ifdef __WXMAC__
+#ifdef __WXOSX__
     wxMenuBar *bar = wxXmlResource::Get()->LoadMenuBar("mainmenu_mac_global");
     TweakOSXMenuBar(bar);
     wxMenuBar::MacSetCommonMenuBar(bar);
@@ -258,32 +262,42 @@ bool PoeditApp::OnInit()
         OpenFiles(gs_filesToOpen);
         gs_filesToOpen.clear();
     }
-#ifndef __WXMAC__
+#ifndef __WXOSX__
     else
     {
         OpenNewFile();
     }
-#endif // !__WXMAC__
+#endif // !__WXOSX__
 
 #ifdef USE_SPARKLE
     Sparkle_Initialize(CheckForBetaUpdates());
 #endif // USE_SPARKLE
 
 #ifdef __WXMSW__
-    const char *appcast = "http://releases.poedit.net/appcast-win";
-
+    wxString appcast = "https://poedit.net/updates/win/appcast";
     if ( CheckForBetaUpdates() )
     {
         // Beta versions use unstable feed.
-        appcast = "http://releases.poedit.net/appcast-win/beta";
+        appcast = "https://poedit.net/updates/win/appcast/beta";
     }
+    if (!wxPlatformInfo().CheckOSVersion(6,0)) // XP doesn't support SNI
+        appcast.Replace("https://", "http://");
 
-    win_sparkle_set_appcast_url(appcast);
+    win_sparkle_set_appcast_url(appcast.utf8_str());
     win_sparkle_set_can_shutdown_callback(&PoeditApp::WinSparkle_CanShutdown);
     win_sparkle_set_shutdown_request_callback(&PoeditApp::WinSparkle_Shutdown);
     win_sparkle_init();
 #endif
 
+#ifndef __WXOSX__
+    // If we failed to open any window during startup (e.g. because the user
+    // attempted to open MO files), shut the app down. Don't do this on OS X
+    // where a) the initialization is finished after OnInit() and b) apps
+    // without windows are OK.
+    if (!PoeditFrame::HasAnyWindow())
+        return false;
+#endif
+
     return true;
 }
 
@@ -313,16 +327,25 @@ static wxLayoutDirection g_layoutDirection = wxLayout_Default;
 void PoeditApp::SetupLanguage()
 {
 #if defined(__WXMSW__)
-	wxLocale::AddCatalogLookupPathPrefix(wxStandardPaths::Get().GetResourcesDir() + "\\Translations");
-#elif !defined(__WXMAC__)
-    wxLocale::AddCatalogLookupPathPrefix(wxStandardPaths::Get().GetInstallPrefix() + "/share/locale");
+	wxFileTranslationsLoader::AddCatalogLookupPathPrefix(wxStandardPaths::Get().GetResourcesDir() + "\\Translations");
+#elif !defined(__WXOSX__)
+    wxFileTranslationsLoader::AddCatalogLookupPathPrefix(wxStandardPaths::Get().GetInstallPrefix() + "/share/locale");
 #endif
 
     wxTranslations *trans = new wxTranslations();
     wxTranslations::Set(trans);
-    #if NEED_CHOOSELANG_UI
+
+#ifdef __WXGTK__
+    // Properly set locale is important for some aspects of GTK+ as well as
+    // other things. It's also the common thing to do, so don't break
+    // expectations needlessly:
+    m_locale.reset(new wxLocale());
+    m_locale->Init(wxLANGUAGE_DEFAULT, wxLOCALE_DONT_LOAD_DEFAULT);
+#endif
+
+#if NEED_CHOOSELANG_UI
     trans->SetLanguage(GetUILanguage());
-    #endif
+#endif
     trans->AddStdCatalog();
     trans->AddCatalog("poedit");
 
@@ -349,45 +372,80 @@ void PoeditApp::OpenNewFile()
 void PoeditApp::OpenFiles(const wxArrayString& names)
 {
     PoeditFrame *active = PoeditFrame::UnusedActiveWindow();
-    if (names.size() == 1 && active)
-    {
-        active->OpenFile(names[0]);
-        return;
-    }
 
     for ( auto name: names )
-        PoeditFrame::Create(name);
+    {
+        // MO files cannot be opened directly in Poedit (yet), but they are
+        // associated with it, so that the app can provide explanation to users
+        // not familiar with the MO/PO distinction.
+        auto n = name.Lower();
+        if (n.EndsWith(".mo") || n.EndsWith(".gmo"))
+        {
+            wxMessageDialog dlg(nullptr,
+                                _("MO files can’t be directly edited in Poedit."),
+                                _("Error opening file"),
+                                wxOK);
+            dlg.SetExtendedMessage(_("Please open and edit the corresponding PO file instead. When you save it, the MO file will be updated as well."));
+            dlg.ShowModal();
+            continue;
+        }
+
+        if (active)
+        {
+            active->OpenFile(name);
+            active = nullptr;
+        }
+        else
+        {
+            PoeditFrame::Create(name);
+        }
+    }
 }
 
-void PoeditApp::SetDefaultParsers(wxConfigBase *cfg)
+void PoeditApp::SetDefaultExtractors(wxConfigBase *cfg)
 {
-    ParsersDB pdb;
+    ExtractorsDB db;
     bool changed = false;
     wxString defaultsVersion = cfg->Read("Parsers/DefaultsVersion",
                                          "1.2.x");
-    pdb.Read(cfg);
+    db.Read(cfg);
 
-    // Add parsers for languages supported by gettext itself (but only if the
+    // Add extractors for languages supported by gettext itself (but only if the
     // user didn't already add language with this name himself):
     static struct
     {
         const char *name;
         const char *exts;
     } s_gettextLangs[] = {
-        { "C/C++",    "*.c;*.cpp;*.h;*.hpp;*.cc;*.C;*.cxx;*.hxx" },
-        { "C#",       "*.cs" },
-        { "Java",     "*.java" },
-        { "Perl",     "*.pl" },
-        { "PHP",      "*.php;*.phtml" },
-        { "Python",   "*.py" },
-        { "TCL",      "*.tcl" },
+        { "C/C++",      "*.c;*.cpp;*.cc;*.C;*.c++;*.cxx;*.h;*.hpp;*.hxx;*.hh" },
+        { "C#",         "*.cs" },
+        { "EmacsLisp",  "*.el" },
+        { "GSettings",  "gschema.xml" },
+        { "Glade",      "*.glade;*.glade2;*.ui" },
+        { "Java",       "*.java" },
+        { "JavaScript", "*.js" },
+        { "Lisp",       "*.lisp" },
+        { "Lua",        "*.lua" },
+        { "ObjectiveC", "*.m" },
+        { "PHP",        "*.php;*.php3;*.php4;*.phtml" },
+        { "Perl",       "*.pl;*.PL;*.pm;*.perl" },
+        { "Python",     "*.py" },
+        { "RST",        "*.rst" },
+        { "Scheme",     "*.scm" },
+        { "Shell",      "*.sh;*.bash" },
+        { "Smalltalk",  "*.st" },
+        { "TCL",        "*.tcl" },
+        { "Vala",       "*.vala" },
+        { "YCP",        "*.ycp" },
+        { "awk",        "*.awk" },
+        { "librep",     "*.jl" },
         { NULL, NULL }
     };
 
     for (size_t i = 0; s_gettextLangs[i].name != NULL; i++)
     {
         // if this lang is already registered, don't overwrite it:
-        if (pdb.FindParser(s_gettextLangs[i].name) != -1)
+        if (db.FindExtractor(s_gettextLangs[i].name) != -1)
             continue;
 
         wxString langflag;
@@ -396,43 +454,43 @@ void PoeditApp::SetDefaultParsers(wxConfigBase *cfg)
         else
             langflag = wxString(" --language=") + s_gettextLangs[i].name;
 
-        // otherwise add new parser:
-        Parser p;
-        p.Name = s_gettextLangs[i].name;
-        p.Extensions = s_gettextLangs[i].exts;
-        p.Command = wxString("xgettext") + langflag + " --add-comments=TRANSLATORS --add-comments=translators: --force-po -o %o %C %K %F";
-        p.KeywordItem = "-k%k";
-        p.FileItem = "%f";
-        p.CharsetItem = "--from-code=%c";
-        pdb.Add(p);
+        // otherwise add new extractor:
+        Extractor ex;
+        ex.Name = s_gettextLangs[i].name;
+        ex.Extensions = s_gettextLangs[i].exts;
+        ex.Command = wxString("xgettext") + langflag + " --add-comments=TRANSLATORS: --force-po -o %o %C %K %F";
+        ex.KeywordItem = "-k%k";
+        ex.FileItem = "%f";
+        ex.CharsetItem = "--from-code=%c";
+        db.Data.push_back(ex);
         changed = true;
     }
 
-    // If upgrading Poedit to 1.2.4, add dxgettext parser for Delphi:
+    // If upgrading Poedit to 1.2.4, add dxgettext extractor for Delphi:
 #ifdef __WINDOWS__
     if (defaultsVersion == "1.2.x")
     {
-        Parser p;
+        Extractor p;
         p.Name = "Delphi (dxgettext)";
         p.Extensions = "*.pas;*.dpr;*.xfm;*.dfm";
         p.Command = "dxgettext --so %o %F";
         p.KeywordItem = wxEmptyString;
         p.FileItem = "%f";
-        pdb.Add(p);
+        db.Data.push_back(p);
         changed = true;
     }
 #endif
 
-    // If upgrading Poedit to 1.2.5, update C++ parser to handle --from-code:
+    // If upgrading Poedit to 1.2.5, update C++ extractor to handle --from-code:
     if (defaultsVersion == "1.2.x" || defaultsVersion == "1.2.4")
     {
-        int cpp = pdb.FindParser("C/C++");
+        int cpp = db.FindExtractor("C/C++");
         if (cpp != -1)
         {
-            if (pdb[cpp].Command == "xgettext --force-po -o %o %K %F")
+            if (db.Data[cpp].Command == "xgettext --force-po -o %o %K %F")
             {
-                pdb[cpp].Command = "xgettext --force-po -o %o %C %K %F";
-                pdb[cpp].CharsetItem = "--from-code=%c";
+                db.Data[cpp].Command = "xgettext --force-po -o %o %C %K %F";
+                db.Data[cpp].CharsetItem = "--from-code=%c";
                 changed = true;
             }
         }
@@ -440,9 +498,9 @@ void PoeditApp::SetDefaultParsers(wxConfigBase *cfg)
 
     // Poedit 1.5.6 had a breakage, it add --add-comments without space in front of it.
     // Repair this automatically:
-    for (size_t i = 0; i < pdb.GetCount(); i++)
+    for (size_t i = 0; i < db.Data.size(); i++)
     {
-        wxString& cmd = pdb[i].Command;
+        wxString& cmd = db.Data[i].Command;
         if (cmd.Contains("--add-comments=") && !cmd.Contains(" --add-comments="))
         {
             cmd.Replace("--add-comments=", " --add-comments=");
@@ -452,14 +510,14 @@ void PoeditApp::SetDefaultParsers(wxConfigBase *cfg)
 
     if (changed)
     {
-        pdb.Write(cfg);
+        db.Write(cfg);
         cfg->Write("Parsers/DefaultsVersion", GetAppVersion());
     }
 }
 
 void PoeditApp::SetDefaultCfg(wxConfigBase *cfg)
 {
-    SetDefaultParsers(cfg);
+    SetDefaultExtractors(cfg);
 
     if (cfg->Read("version", wxEmptyString) == GetAppVersion()) return;
 
@@ -519,18 +577,14 @@ bool PoeditApp::OnExceptionInMainLoop()
     {
         throw;
     }
-    catch ( Exception& e )
+    catch ( std::exception& )
     {
-        wxLogError(_("Unhandled exception occurred: %s"), e.what());
-    }
-    catch ( std::exception& e )
-    {
-        wxLogError(_("Unhandled exception occurred: %s"), e.what());
+        wxLogError(_("Unhandled exception occurred: %s"), DescribeCurrentException());
     }
 #ifdef __WXOSX__
     catch ( NSException *e )
     {
-        wxLogError(_("Unhandled exception occurred: %s"), wxStringWithNSString([e reason]));
+        wxLogError(_("Unhandled exception occurred: %s"), wxStringFromNS([e reason]));
     }
 #endif
     catch ( ... )
@@ -566,6 +620,7 @@ BEGIN_EVENT_TABLE(PoeditApp, wxApp)
    EVT_MENU           (XRCID("menu_check_for_updates"), PoeditApp::OnWinsparkleCheck)
 #endif
 #ifdef __WXOSX__
+   EVT_MENU           (wxID_CLOSE, PoeditApp::OnCloseWindowCommand)
    EVT_IDLE           (PoeditApp::OnIdleInstallOpenRecentMenu)
 #endif
 END_EVENT_TABLE()
@@ -605,7 +660,8 @@ void PoeditApp::OnOpen(wxCommandEvent&)
                      _("Open catalog"),
                      path,
                      wxEmptyString,
-                     _("GNU gettext catalogs (*.po)|*.po|All files (*.*)|*.*"),
+                     wxString::Format("%s (*.po)|*.po|%s (*.*)|*.*",
+                         _("PO Translation Files"), _("All Files")),
                      wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_MULTIPLE);
 
     if (dlg.ShowModal() == wxID_OK)
@@ -661,12 +717,12 @@ void PoeditApp::OnAbout(wxCommandEvent&)
 
     wxAboutDialogInfo about;
 
-#ifndef __WXMAC__
+#ifndef __WXOSX__
     about.SetName("Poedit");
     about.SetVersion(wxGetApp().GetAppVersion());
     about.SetDescription(_("Poedit is an easy to use translations editor."));
 #endif
-    about.SetCopyright(L"Copyright \u00a9 1999-2014 Václav Slavík");
+    about.SetCopyright(L"Copyright \u00a9 1999-2015 Václav Slavík");
 #ifdef __WXGTK__ // other ports would show non-native about dlg
     about.SetWebSite("http://poedit.net");
 #endif
@@ -706,14 +762,9 @@ void PoeditApp::OnQuit(wxCommandEvent&)
 
 void PoeditApp::EditPreferences()
 {
-    PreferencesDialog dlg(NULL);
-
-    dlg.TransferTo(wxConfig::Get());
-    if (dlg.ShowModal() == wxID_OK)
-    {
-        dlg.TransferFrom(wxConfig::Get());
-        PoeditFrame::UpdateAllAfterPreferencesChange();
-    }
+    if (!m_preferences)
+        m_preferences = PoeditPreferencesEditor::Create();
+    m_preferences->Show(nullptr);
 }
 
 void PoeditApp::OnPreferences(wxCommandEvent&)
@@ -747,7 +798,7 @@ void PoeditApp::OpenPoeditWeb(const wxString& path)
 
 static NSMenuItem *AddNativeItem(NSMenu *menu, int pos, const wxString&text, SEL ac, NSString *key)
 {
-    NSString *str = wxNSStringWithWxString(text);
+    NSString *str = wxStringToNS(text);
     if (pos == -1)
         return [menu addItemWithTitle:str action:ac keyEquivalent:key];
     else
@@ -767,7 +818,7 @@ void PoeditApp::TweakOSXMenuBar(wxMenuBar *bar)
     Sparkle_AddMenuItem(apple->GetHMenu(), _("Check for Updates...").utf8_str());
 #endif
 
-    int editMenuPos = bar->FindMenu(_("Edit"));
+    int editMenuPos = bar->FindMenu(_("&Edit"));
     if (editMenuPos == wxNOT_FOUND)
         editMenuPos = 1;
     wxMenu *edit = bar->GetMenu(editMenuPos);
@@ -840,6 +891,16 @@ void PoeditApp::TweakOSXMenuBar(wxMenuBar *bar)
     AddNativeItem(speech, -1, _("Stop Speaking"), @selector(stopSpeaking:), @"");
     [editNS setSubmenu:speech forItem:item];
 
+    int viewMenuPos = bar->FindMenu(_("&View"));
+    if (viewMenuPos != wxNOT_FOUND)
+    {
+        NSMenu *viewNS = bar->GetMenu(viewMenuPos)->GetHMenu();
+        [viewNS addItem:[NSMenuItem separatorItem]];
+        item = AddNativeItem(viewNS, -1, _("Enter Full Screen"), @selector(toggleFullScreen:), @"f");
+        [item setKeyEquivalentModifierMask:NSCommandKeyMask | NSControlKeyMask];
+
+    }
+
     int windowMenuPos = bar->FindMenu(_("Window"));
     if (windowMenuPos != wxNOT_FOUND)
     {
@@ -889,7 +950,7 @@ void PoeditApp::InstallOpenRecentMenu(wxMenuBar *bar)
         return;
 
     NSMenu *native = fileMenu->GetHMenu();
-    NSMenuItem *nativeItem = [native itemWithTitle:wxNSStringWithWxString(item->GetItemLabelText())];
+    NSMenuItem *nativeItem = [native itemWithTitle:wxStringToNS(item->GetItemLabelText())];
     if (!nativeItem)
         return;
 
@@ -912,6 +973,19 @@ void PoeditApp::OSXOnWillFinishLaunching()
     CreateFakeOpenRecentMenu();
 }
 
+// Handle Cmd+W closure of windows globally here
+void PoeditApp::OnCloseWindowCommand(wxCommandEvent&)
+{
+    for (auto w: wxTopLevelWindows)
+    {
+        auto tlw = dynamic_cast<wxTopLevelWindow*>(w);
+        if (tlw && tlw->IsActive())
+        {
+            tlw->Close();
+            break;
+        }
+    }
+}
 
 #endif // __WXOSX__
 
@@ -937,14 +1011,14 @@ void PoeditApp::AssociateFileTypeIfNeeded()
     auto poIcon = wxStandardPaths::Get().GetResourcesDir() + "\\Resources\\poedit-translation-generic.ico";
     wxRegKey key1(wxRegKey::HKCU, "Software\\Classes\\.po");
     key1.Create();
-    key1.SetValue("", "GettextFile");
-    wxRegKey key2(wxRegKey::HKCU, "Software\\Classes\\GettextFile");
+    key1.SetValue("", "Poedit.PO");
+    wxRegKey key2(wxRegKey::HKCU, "Software\\Classes\\Poedit.PO");
     key2.Create();
-    key2.SetValue("", _("PO Translation File"));
-    wxRegKey key3(wxRegKey::HKCU, "Software\\Classes\\GettextFile\\Shell\\Open\\Command");
+    key2.SetValue("", _("PO Translation"));
+    wxRegKey key3(wxRegKey::HKCU, "Software\\Classes\\Poedit.PO\\Shell\\Open\\Command");
     key3.Create();
     key3.SetValue("", poCmd);
-    wxRegKey key4(wxRegKey::HKCU, "Software\\Classes\\GettextFile\\DefaultIcon");
+    wxRegKey key4(wxRegKey::HKCU, "Software\\Classes\\Poedit.PO\\DefaultIcon");
     key4.Create();
     key4.SetValue("", poIcon);
 }
