@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2001-2014 Vaclav Slavik
+ *  Copyright (C) 2001-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -54,16 +54,12 @@ FindFrame::FindFrame(wxWindow *parent,
                      PoeditListCtrl *list,
                      Catalog *c,
                      wxTextCtrl *textCtrlOrig,
-                     wxTextCtrl *textCtrlTrans,
-                     wxTextCtrl *textCtrlComments,
-                     wxTextCtrl *textCtrlAutoComments)
+                     wxTextCtrl *textCtrlTrans)
         : m_listCtrl(list),
           m_catalog(c),
           m_position(-1),
           m_textCtrlOrig(textCtrlOrig),
-          m_textCtrlTrans(textCtrlTrans),
-          m_textCtrlComments(textCtrlComments),
-          m_textCtrlAutoComments(textCtrlAutoComments)
+          m_textCtrlTrans(textCtrlTrans)
 {
     wxXmlResource::Get()->LoadDialog(this, parent, "find_frame");
 
@@ -240,7 +236,7 @@ enum FoundState
     Found_InOrig,
     Found_InTrans,
     Found_InComments,
-    Found_InAutoComments
+    Found_InExtractedComments
 };
 
 bool TextInString(const wxString& str, const wxString& text, bool wholeWords)
@@ -280,7 +276,7 @@ bool FindFrame::DoFind(int dir)
     bool inStr = XRCCTRL(*this, "in_orig", wxCheckBox)->GetValue();
     bool inTrans = XRCCTRL(*this, "in_trans", wxCheckBox)->GetValue();
     bool inComments = XRCCTRL(*this, "in_comments", wxCheckBox)->GetValue();
-    bool inAutoComments = XRCCTRL(*this, "in_auto_comments", wxCheckBox)->GetValue();
+    bool inExtractedComments = XRCCTRL(*this, "in_auto_comments", wxCheckBox)->GetValue();
     bool caseSens = XRCCTRL(*this, "case_sensitive", wxCheckBox)->GetValue();
     bool wholeWords = XRCCTRL(*this, "whole_words", wxCheckBox)->GetValue();
     int posOrig = m_position;
@@ -346,17 +342,17 @@ bool FindFrame::DoFind(int dir)
 
             if (TextInString(textc, text, wholeWords)) { found = Found_InComments; break; }
         }
-        if (inAutoComments)
+        if (inExtractedComments)
         {
-            wxArrayString autoComments = dt.GetAutoComments();
+            wxArrayString extractedComments = dt.GetExtractedComments();
             textc = wxEmptyString;
-            for (unsigned i = 0; i < autoComments.GetCount(); i++)
-                textc += autoComments[i];
+            for (unsigned i = 0; i < extractedComments.GetCount(); i++)
+                textc += extractedComments[i];
 
             if (!caseSens)
                 textc.MakeLower();
 
-            if (TextInString(textc, text, wholeWords)) { found = Found_InAutoComments; break; }
+            if (TextInString(textc, text, wholeWords)) { found = Found_InExtractedComments; break; }
         }
 
         m_position += dir;
@@ -365,12 +361,10 @@ bool FindFrame::DoFind(int dir)
     if (found != Found_Not)
     {
         m_listCtrl->EnsureVisible(m_position);
-#ifdef __WXMAC__
+#ifdef __WXOSX__
         m_listCtrl->Refresh();
 #endif
-        m_listCtrl->Select(m_position);
-        m_listCtrl->SetItemState(m_position,
-                    wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+        m_listCtrl->SelectAndFocus(m_position);
 
         // find the text on the control and select it:
 
@@ -384,21 +378,20 @@ bool FindFrame::DoFind(int dir)
               txt = m_textCtrlTrans;
               break;
             case Found_InComments:
-              txt = m_textCtrlComments;
-              break;
-            case Found_InAutoComments:
-              txt = m_textCtrlAutoComments;
-              break;
-            case Found_Not: // silence compiler warning, can't get here
+            case Found_InExtractedComments:
+            case Found_Not:
               break;
         }
 
-        textc = txt->GetValue();
-        if (!caseSens)
-            textc.MakeLower();
-        int pos = textc.Find(text);
-        if (pos != wxNOT_FOUND)
-            txt->SetSelection(pos, pos + text.length());
+        if (txt)
+        {
+            textc = txt->GetValue();
+            if (!caseSens)
+                textc.MakeLower();
+            int pos = textc.Find(text);
+            if (pos != wxNOT_FOUND)
+                txt->SetSelection(pos, pos + text.length());
+        }
 
         return true;
     }
