@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2001-2014 Vaclav Slavik
+ *  Copyright (C) 2001-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -51,8 +51,7 @@ class FindFrame : public wxDialog
             \param catalog Catalog to search in
          */
         FindFrame(wxWindow *parent, PoeditListCtrl *list, Catalog *c,
-                  wxTextCtrl *textCtrlOrig, wxTextCtrl *textCtrlTrans,
-                  wxTextCtrl *textCtrlComments, wxTextCtrl *textCtrlAutoComments);
+                  wxTextCtrl *textCtrlOrig, wxTextCtrl *textCtrlTrans);
         ~FindFrame();
 
         /// Returns singleton instance if it exists. Updates the catalog
@@ -88,7 +87,6 @@ class FindFrame : public wxDialog
         int m_position;
         wxButton *m_btnPrev, *m_btnNext;
         wxTextCtrl *m_textCtrlOrig, *m_textCtrlTrans;
-        wxTextCtrl *m_textCtrlComments, *m_textCtrlAutoComments;
 
         // NB: this is static so that last search term is remembered
         static wxString ms_text;
