@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2001-2014 Vaclav Slavik
+ *  Copyright (C) 2001-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -49,6 +49,7 @@
 #include "catalog.h"
 #include "edapp.h"
 #include "edframe.h"
+#include "hidpi.h"
 #include "manager.h"
 #include "progressinfo.h"
 #include "utility.h"
@@ -93,7 +94,7 @@ ManagerFrame::ManagerFrame() :
     m_listCat = XRCCTRL(*panel, "prj_files", wxListCtrl);
     m_splitter = XRCCTRL(*panel, "manager_splitter", wxSplitterWindow);
 
-    wxImageList *list = new wxImageList(16, 16);
+    wxImageList *list = new wxImageList(PX(16), PX(16));
     list->Add(wxArtProvider::GetBitmap("poedit-status-cat-no"));
     list->Add(wxArtProvider::GetBitmap("poedit-status-cat-mid"));
     list->Add(wxArtProvider::GetBitmap("poedit-status-cat-ok"));
@@ -108,9 +109,9 @@ ManagerFrame::ManagerFrame() :
     if (m_listPrj->GetCount() > 0)
         UpdateListCat(last);
 
-    RestoreWindowState(this, wxSize(400, 300));
+    RestoreWindowState(this, wxSize(PX(400), PX(300)));
 
-    m_splitter->SetSashPosition((int)wxConfig::Get()->Read("manager_splitter", 200));
+    m_splitter->SetSashPosition((int)wxConfig::Get()->Read("manager_splitter", PX(200)));
 }
 
 
@@ -457,7 +458,8 @@ void ManagerFrame::OnUpdateProject(wxCommandEvent&)
             else
             {
                 Catalog cat(f);
-                cat.Update(&pinfo, /*summary=*/false, nullptr);
+                UpdateResultReason reason;
+                cat.Update(&pinfo, /*summary=*/false, reason);
                 int validation_errors = 0;
                 Catalog::CompilationStatus mo_status;
                 cat.Save(f, false, validation_errors, mo_status);
