@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2010-2014 Vaclav Slavik
+ *  Copyright (C) 2010-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -36,8 +36,24 @@
 #include <wx/arrstr.h>
 #include <wx/toplevel.h>
 
+// ----------------------------------------------------------------------
+// Misc platform differences
+// ----------------------------------------------------------------------
+
+#ifdef __WXMSW__
+    #define MSW_OR_OTHER(msw, other) msw
+#else
+    #define MSW_OR_OTHER(msw, other) other
+#endif
+
+
+// ----------------------------------------------------------------------
+// Misc helpers
+// ----------------------------------------------------------------------
+
 wxString EscapeMarkup(const wxString& str);
 
+
 // ----------------------------------------------------------------------
 // TempDirectory
 // ----------------------------------------------------------------------
